Imports Benday.Npr.Phone.Common.Repositories
Imports Benday.Npr.Phone.Common.Models
Imports Benday.Npr.Phone.Common
Imports System.Net
Imports System.IO

Namespace Benday.Npr.Phone.DataAccess
	Public Class StoryRepository
		Inherits RepositoryBase
		Implements IStoryRepository

        Public Event StoryListRefreshed As EventHandler(Of StoryListRefreshedEventArgs) Implements IStoryRepository.StoryListRefreshed

		Public Property TestModeConfiguration As StoryRepositoryTestModeConfiguration

		Public Sub New(ByVal factory As IObjectFactory, ByVal adapter As IStoryXmlAdapter)
			MyBase.New(factory)
			If adapter Is Nothing Then
				Throw New ArgumentNullException("adapter", "adapter is null.")
			End If

			m_AdapterInstance = adapter
		End Sub

		Private m_AdapterInstance As IStoryXmlAdapter

        Public Function GetStories(ByVal programId As String) As IStoryList Implements IStoryRepository.GetStories
            Dim returnValue = ObjectFactoryInstance.Create(Of IStoryList)()

            Dim fileContents = DataAccessUtility.ReadFile(DataAccessUtility.GetProgramStoriesFilename(programId))

            If String.IsNullOrEmpty(fileContents) = False Then
                ' turn the cached values into objects and return
                returnValue = ProcessStories(programId, fileContents)
            Else
                ' no cached stories for this program id
                RefreshStories(programId)
            End If

            Return returnValue
        End Function

        Public Sub RefreshStories(ByVal programId As String) Implements IStoryRepository.RefreshStories
            If String.IsNullOrEmpty(programId) Then
                Throw New ArgumentException("programId is null or empty.", "programId")
            End If

            Dim action = New RefreshStoriesReturnResultAction(New ReturnResult(Of IStoryList), programId)

            If programId.StartsWith("RSS_FEED.") = True Then
                ' RSS_FEED.ITS_ALL_POLITICS
                ' RSS_FEED.NPR_HOURLY_NEWS
                GetStoriesForRssFeed(action, programId)
            Else
                GetStoriesForProgram(action, programId)
            End If
        End Sub

		Private Sub GetStoriesForRssFeed(ByVal action As RefreshStoriesReturnResultAction, ByVal id As String)
			Dim url As String

			If id = "RSS_FEED.NPR_HOURLY_NEWS" Then
				url = "http://www.npr.org/rss/podcast.php?id=500005"
			ElseIf id = "RSS_FEED.ITS_ALL_POLITICS" Then
				url = "http://www.npr.org/rss/podcast.php?id=510068"
			ElseIf id = "RSS_FEED.CAR_TALK" Then
				url = "http://www.npr.org/rss/podcast.php?id=510208"
			Else
				Throw New InvalidOperationException(String.Format("Unknown rss feed id '{0}'.", id))
			End If

			Dim request As HttpWebRequest = TryCast(WebRequest.Create(url), HttpWebRequest)

			request.BeginGetResponse(AddressOf ProcessStoriesFromRssFeedResult, New RefreshStoriesReturnResult(request, action))
		End Sub

		Private Sub GetStoriesForProgram(ByVal action As RefreshStoriesReturnResultAction, ByVal id As String)
			If TestModeConfiguration IsNot Nothing Then
				ProcessStories(action, TestModeConfiguration.TestData)
			Else
				Dim url As String = String.Format("http://api.npr.org/query?id={0}&apiKey=MDA2MzU3ODU0MDEyODg2MTQyOTk1OWQ4NQ001", id)

				Dim request As HttpWebRequest = TryCast(WebRequest.Create(url), HttpWebRequest)

				request.BeginGetResponse(AddressOf ProcessStoriesFromQueryResult, New RefreshStoriesReturnResult(request, action))
			End If
		End Sub

		Private Sub WriteResponseToDisk(ByVal programId As String, ByVal writeThisToDisk As String)
			Dim filename = DataAccessUtility.GetProgramStoriesFilename(programId)

			DataAccessUtility.SaveFile(filename, writeThisToDisk)
		End Sub

		Private Function ProcessStories(ByVal programId As String, ByVal xml As String) As IStoryList
			If programId.StartsWith("RSS_FEED.") = True Then
				' RSS_FEED.ITS_ALL_POLITICS
				' RSS_FEED.NPR_HOURLY_NEWS
				Dim adapter = ObjectFactoryInstance.Create(Of IRssFeedXmlAdapter)()

				Dim returnValue = ObjectFactoryInstance.Create(Of IStoryList)()

				adapter.Adapt(xml, returnValue)

				WriteResponseToDisk(programId, xml)

				Return returnValue
			Else
				Dim adapter = ObjectFactoryInstance.Create(Of IStoryXmlAdapter)()

				Dim returnValue = ObjectFactoryInstance.Create(Of IStoryList)()

				adapter.Adapt(xml, returnValue)

				WriteResponseToDisk(programId, xml)

				Return returnValue
			End If
		End Function

		Private Sub ProcessStories(ByVal action As RefreshStoriesReturnResultAction, ByVal xml As String)
			Dim returnValue As IStoryList = ProcessStories(action.ProgramId, xml)

            System.Windows.Deployment.Current.Dispatcher.BeginInvoke(Sub() action.Action.Notify(returnValue))

			System.Windows.Deployment.Current.Dispatcher.BeginInvoke(Sub() OnStoryListRefreshed(New StoryListRefreshedEventArgs(action.ProgramId)))
		End Sub

		Private Sub ProcessStoriesFromQueryResult(ByVal asyncResult As IAsyncResult)
			Dim action = TryCast(asyncResult.AsyncState, RefreshStoriesReturnResult)

			If action Is Nothing Then
				Throw New InvalidOperationException("User state was the wrong type.")
			End If

			Try
				Dim response = action.Request.EndGetResponse(asyncResult)

				Using stream = New StreamReader(response.GetResponseStream)
					Dim xml As String = stream.ReadToEnd

					ProcessStories(action.ReturnResult, xml)
				End Using
			Catch e1 As Exception
                System.Windows.Deployment.Current.Dispatcher.BeginInvoke(Sub() action.ReturnResult.Action.Notify(TryCast(Nothing, IStoryList)))
			End Try
		End Sub

		Private Sub ProcessStoriesFromRssFeedResult(ByVal asyncResult As IAsyncResult)
			Dim action = TryCast(asyncResult.AsyncState, RefreshStoriesReturnResult)

			If action Is Nothing Then
				Throw New InvalidOperationException("User state was the wrong type.")
			End If

			Try
				Dim response = action.Request.EndGetResponse(asyncResult)

				Using stream = New StreamReader(response.GetResponseStream)
					Dim xml As String = stream.ReadToEnd

					Dim adapter = ObjectFactoryInstance.Create(Of IRssFeedXmlAdapter)()

					Dim returnValue = ObjectFactoryInstance.Create(Of IStoryList)()

					adapter.Adapt(xml, returnValue)

					WriteResponseToDisk(action.ReturnResult.ProgramId, xml)

                    System.Windows.Deployment.Current.Dispatcher.BeginInvoke(Sub() action.ReturnResult.Action.Notify(returnValue))

					System.Windows.Deployment.Current.Dispatcher.BeginInvoke(Sub() OnStoryListRefreshed(New StoryListRefreshedEventArgs(action.ReturnResult.ProgramId)))
				End Using
			Catch e1 As Exception
                System.Windows.Deployment.Current.Dispatcher.BeginInvoke(Sub() action.ReturnResult.Action.Notify(TryCast(Nothing, IStoryList)))
			End Try
		End Sub

		Private Function GetStoryNotFoundStory(ByVal storyId As String) As IStory
			Dim story = ObjectFactoryInstance.Create(Of IStory)()

			story.Id = storyId
			story.Title = "Story not found"

			Return story
		End Function

        Public Function GetStory(ByVal programId As String, ByVal storyId As String) As IStory Implements IStoryRepository.GetStory
            Dim stories = GetStories(programId)

            If stories Is Nothing OrElse stories.Stories Is Nothing OrElse stories.Stories.Count = 0 Then
                Return GetStoryNotFoundStory(storyId)
            Else
                Dim matchingStory As IStory = Nothing

                If storyId = "RSS_FEED.NPR_HOURLY_NEWS" Then
                    matchingStory = (
                        From temp In stories.Stories
                        Select temp).FirstOrDefault
                Else
                    matchingStory = (
                        From temp In stories.Stories
                        Where temp.Id = storyId
                        Select temp).FirstOrDefault
                End If

                If matchingStory Is Nothing Then
                    Return GetStoryNotFoundStory(storyId)
                Else
                    Return matchingStory
                End If
            End If
        End Function

        Public Sub SaveCurrentStoryId(ByVal currentId As String) Implements IStoryRepository.SaveCurrentStoryId
            Dim doc = XmlUtility.StringToXDocument("<info />")

            If currentId Is Nothing Then
                currentId = String.Empty
            Else
                currentId = currentId.Trim
            End If

            XmlUtility.SetChildElement(doc.Root, "CurrentStoryId", currentId)

            DataAccessUtility.SaveFile(DataAccessConstants.FilenameStoryInformation, doc)
        End Sub

        Public Function GetCurrentStoryId() As String Implements IStoryRepository.GetCurrentStoryId
            Dim contents = DataAccessUtility.ReadFile(DataAccessConstants.FilenameStoryInformation)

            If contents Is Nothing Then
                Return Nothing
            Else
                Dim doc = XmlUtility.StringToXDocument(contents)

                Dim returnValue As String = XmlUtility.GetChildElementValue(doc.Root, "CurrentStoryId", CType(Nothing, String))

                If returnValue Is Nothing OrElse returnValue = String.Empty Then
                    Return Nothing
                Else
                    Return returnValue
                End If
            End If
        End Function

		Public Overridable Sub OnStoryListRefreshed(ByVal ea As StoryListRefreshedEventArgs)
			RaiseEvent StoryListRefreshed(Me, ea)
		End Sub

		Private Sub ProcessMp3ResolveResult(ByVal asyncResult As IAsyncResult)
			Dim action = TryCast(asyncResult.AsyncState, WebRequestReturnResult(Of String))

			If action Is Nothing Then
				Throw New InvalidOperationException("User state was the wrong type.")
			End If

			Try
				Dim response = action.Request.EndGetResponse(asyncResult)

				Using stream = New StreamReader(response.GetResponseStream)
					Dim m3uPlaylistContents As String = stream.ReadToEnd

					Dim playlistReader As New StringReader(m3uPlaylistContents)

					Dim urlToMp3File As String = playlistReader.ReadLine

					Do While urlToMp3File IsNot Nothing AndAlso urlToMp3File.StartsWith("http://") = False
						urlToMp3File = playlistReader.ReadLine
					Loop

                    System.Windows.Deployment.Current.Dispatcher.BeginInvoke(Sub() action.ReturnResult.Notify(urlToMp3File))
				End Using
			Catch ex As Exception
                System.Windows.Deployment.Current.Dispatcher.BeginInvoke(Sub() action.ReturnResult.Notify(ex))
			End Try
		End Sub

        Public Sub ResolveMp3UrlFromM3u(ByVal action As ReturnResult(Of String), ByVal m3uUri As String) Implements IStoryRepository.ResolveMp3UrlFromM3u
            If String.IsNullOrEmpty(m3uUri) Then
                Throw New ArgumentNullException("m3uUri", "m3uUri is null.")
            End If
            If action Is Nothing Then
                Throw New ArgumentNullException("action", "action is null.")
            End If

            Try
                Dim request As HttpWebRequest = TryCast(WebRequest.Create(m3uUri), HttpWebRequest)

                request.BeginGetResponse(AddressOf ProcessMp3ResolveResult, New WebRequestReturnResult(Of String)(request, action))
            Catch e1 As Exception
                Throw
            End Try
        End Sub
	End Class
End Namespace
