Imports Benday.Npr.Phone.Presentation.Interfaces
Imports Benday.Npr.Phone.Common.Models
Imports Benday.Npr.Phone.Common
Imports Benday.Npr.Phone.Common.Repositories

Namespace Benday.Npr.Phone.Presentation
	Public Class StoryListViewModel
		Inherits ObjectFactoryViewModel
        Implements IStoryListViewModel


        Public Sub New(ByVal programRepositoryInstance As IProgramRepository, ByVal storyRepositoryInstance As IStoryRepository, ByVal factory As IObjectFactory)
            MyBase.New(factory)
            If programRepositoryInstance Is Nothing Then
                Throw New ArgumentNullException("programRepositoryInstance", "programRepositoryInstance is null.")
            End If
            If storyRepositoryInstance Is Nothing Then
                Throw New ArgumentNullException("storyRepositoryInstance", "storyRepositoryInstance is null.")
            End If
            If factory Is Nothing Then
                Throw New ArgumentNullException("factory", "factory is null.")
            End If

            m_Stories = New SingleSelectListViewModel(New List(Of ISelectableItem))
            AddHandler m_Stories.OnItemSelected, AddressOf m_Stories_OnItemSelected

            m_StoryRepositoryInstance = storyRepositoryInstance
            AddHandler m_StoryRepositoryInstance.StoryListRefreshed, AddressOf m_StoryRepositoryInstance_StoryListRefreshed

            m_ProgramRepositoryInstance = programRepositoryInstance
        End Sub

        Private Sub m_StoryRepositoryInstance_StoryListRefreshed(ByVal sender As Object, ByVal e As StoryListRefreshedEventArgs)
            Load()
        End Sub

        Private Sub m_Stories_OnItemSelected(ByVal sender As Object, ByVal e As EventArgs)
            HandleStorySelected()
        End Sub

        Private Sub HandleStorySelected()
            If Stories.SelectedItem Is Nothing Then
                CurrentStoryId = Nothing
                StoryRepositoryInstance.SaveCurrentStoryId(Nothing)
            Else
                CurrentStoryId = Stories.SelectedItem.Id
                StoryRepositoryInstance.SaveCurrentStoryId(CurrentStoryId)
            End If
        End Sub

        Private m_ProgramRepositoryInstance As IProgramRepository
        Private ReadOnly Property ProgramRepositoryInstance As IProgramRepository
            Get
                Return m_ProgramRepositoryInstance
            End Get
        End Property

        Private m_StoryRepositoryInstance As IStoryRepository
        Private ReadOnly Property StoryRepositoryInstance As IStoryRepository
            Get
                Return m_StoryRepositoryInstance
            End Get
        End Property

        Public Sub Load() Implements IStoryListViewModel.Load
            CurrentStoryId = StoryRepositoryInstance.GetCurrentStoryId

            PopulateProgramInfo()
            IsProgressBarVisible = True
            ProgressBarMessage = "Loading stories..."

            Id = CurrentProgramId

            Dim stories = StoryRepositoryInstance.GetStories(CurrentProgramId)

            Populate(stories)
        End Sub

        Public Sub Refresh() Implements IStoryListViewModel.Refresh
            StoryRepositoryInstance.RefreshStories(CurrentProgramId)
        End Sub

        Private Sub PopulateProgramInfo()
            Dim programId As String = ProgramRepositoryInstance.GetCurrentProgramId

            If programId IsNot Nothing Then
                Dim program = ProgramRepositoryInstance.GetById(programId)

                If program Is Nothing Then
                    CurrentProgramId = Nothing
                    CurrentProgramTitle = Nothing
                Else
                    CurrentProgramId = programId
                    CurrentProgramTitle = program.Name
                End If
            End If
        End Sub

        Private Sub Populate(ByVal fromValues As IStoryList)
            ProgressBarMessage = "Parsing stories..."

            Try
                Me.Stories.Clear()

                If fromValues Is Nothing Then
                    Title = String.Empty
                    Teaser = String.Empty
                    TeaserMini = String.Empty
                    UrlToApi = String.Empty
                    UrlToHtml = String.Empty

                    RequestMessageBox("That's weird.  We didn't get anything back from NPR.  Any chance you just lost your internet connection?")
                Else
                    Title = fromValues.Title
                    Teaser = fromValues.Teaser
                    TeaserMini = fromValues.TeaserMini
                    UrlToApi = fromValues.UrlToApi
                    UrlToHtml = fromValues.UrlToHtml

                    Dim toValue As SelectableItem = Nothing

                    For Each fromValue In fromValues.Stories
                        toValue = New StorySelectableItem(False, fromValue.Id, fromValue.Title, fromValue.Id, fromValue.Teaser, fromValue.HasUrlToMp3, fromValue.PublishDate)

                        Me.Stories.Add(toValue)
                    Next fromValue
                End If
            Finally
                IsProgressBarVisible = False
            End Try
        End Sub

        Private Const IdPropertyName As String = "Id"

        Private m_Id As String
        Public Property Id As String
            Get
                Return m_Id
            End Get
            Set(ByVal value As String)
                m_Id = value
                RaisePropertyChanged(IdPropertyName)
            End Set
        End Property

        Private Const CurrentStoryIdPropertyName As String = "CurrentStoryId"

        Private m_CurrentStoryId As String
        Public Property CurrentStoryId As String Implements IStoryListViewModel.CurrentStoryId
            Get
                Return m_CurrentStoryId
            End Get
            Set(ByVal value As String)
                m_CurrentStoryId = value
                RaisePropertyChanged(CurrentStoryIdPropertyName)
            End Set
        End Property

        Private Const CurrentProgramIdPropertyName As String = "CurrentProgramId"

        Private m_CurrentProgramId As String
        Public Property CurrentProgramId As String Implements IStoryListViewModel.CurrentProgramId
            Get
                Return m_CurrentProgramId
            End Get
            Set(ByVal value As String)
                m_CurrentProgramId = value
                RaisePropertyChanged(CurrentProgramIdPropertyName)
            End Set
        End Property

        Private Const CurrentProgramTitlePropertyName As String = "CurrentProgramTitle"

        Private m_CurrentProgramTitle As String
        Public Property CurrentProgramTitle As String Implements IStoryListViewModel.CurrentProgramTitle
            Get
                Return m_CurrentProgramTitle
            End Get
            Set(ByVal value As String)
                m_CurrentProgramTitle = value
                RaisePropertyChanged(CurrentProgramTitlePropertyName)
            End Set
        End Property

        Private Const TitlePropertyName As String = "Title"

        Private m_Title As String
        Public Property Title As String
            Get
                Return m_Title
            End Get
            Set(ByVal value As String)
                m_Title = value
                RaisePropertyChanged(TitlePropertyName)
            End Set
        End Property

        Private m_Stories As SingleSelectListViewModel


        Private Const TeaserPropertyName As String = "Teaser"

        Private m_Teaser As String
        Public Property Teaser As String
            Get
                Return m_Teaser
            End Get
            Set(ByVal value As String)
                m_Teaser = value
                RaisePropertyChanged(TeaserPropertyName)
            End Set
        End Property

        Private Const TeaserMiniPropertyName As String = "TeaserMini"

        Private m_TeaserMini As String
        Public Property TeaserMini As String
            Get
                Return m_TeaserMini
            End Get
            Set(ByVal value As String)
                m_TeaserMini = value
                RaisePropertyChanged(TeaserMiniPropertyName)
            End Set
        End Property

        Private Const UrlToHtmlPropertyName As String = "UrlToHtml"

        Private m_UrlToHtml As String
        Public Property UrlToHtml As String
            Get
                Return m_UrlToHtml
            End Get
            Set(ByVal value As String)
                m_UrlToHtml = value
                RaisePropertyChanged(UrlToHtmlPropertyName)
            End Set
        End Property

        Private Const UrlToApiPropertyName As String = "UrlToApi"

        Private m_UrlToApi As String
        Public Property UrlToApi As String
            Get
                Return m_UrlToApi
            End Get
            Set(ByVal value As String)
                m_UrlToApi = value
                RaisePropertyChanged(UrlToApiPropertyName)
            End Set
        End Property

        Private Const IsProgressBarVisiblePropertyName As String = "IsProgressBarVisible"

        Private m_IsProgressBarVisible As Boolean
        Public Property IsProgressBarVisible As Boolean
            Get
                Return m_IsProgressBarVisible
            End Get
            Set(ByVal value As Boolean)
                m_IsProgressBarVisible = value
                RaisePropertyChanged(IsProgressBarVisiblePropertyName)
            End Set
        End Property

        Private Const ProgressBarMessagePropertyName As String = "ProgressBarMessage"

        Private m_ProgressBarMessage As String
        Public Property ProgressBarMessage As String
            Get
                Return m_ProgressBarMessage
            End Get
            Set(ByVal value As String)
                m_ProgressBarMessage = value
                RaisePropertyChanged(ProgressBarMessagePropertyName)
            End Set
        End Property

        Public Property Stories As SingleSelectListViewModel Implements Interfaces.IStoryListViewModel.Stories
            Get

                Return m_Stories

            End Get
            Set(value As SingleSelectListViewModel)
                m_Stories = value
            End Set
        End Property
    End Class
End Namespace
