Imports Microsoft.VisualStudio.TestTools.UnitTesting
Imports Benday.Npr.Phone.DataAccess
Imports Benday.Npr.Phone.Common.Repositories
Imports System.Threading
Imports Microsoft.Silverlight.Testing
Imports Benday.Npr.Phone.Common.Models

Namespace Benday.Npr.Phone.UnitTests.DataAccess
	<TestClass>
	Public Class StoryRepositoryFixture
		Inherits UnitTestBase

		Public Sub New()
		End Sub

		Protected Overrides Sub OnTestInitialize()
			UnitTestUtility.WipeProgramDataInIsolatedStorage()

			m_RepositoryInstance = Nothing
			m_ProgramRepositoryInstance = Nothing
		End Sub

		Private m_RepositoryInstance As StoryRepository
		Public ReadOnly Property RepositoryInstance As StoryRepository
			Get
				If m_RepositoryInstance Is Nothing Then
					m_RepositoryInstance = TryCast(ObjectFactoryInstance.Create(Of IStoryRepository)(), StoryRepository)

					m_RepositoryInstance.TestModeConfiguration = New StoryRepositoryTestModeConfiguration With {.TestData = UnitTestResources.AllThingsConsideredStories}
				End If

				Return m_RepositoryInstance
			End Get
		End Property

		Private m_ProgramRepositoryInstance As ProgramRepository
		Public ReadOnly Property ProgramRepositoryInstance As ProgramRepository
			Get
				If m_ProgramRepositoryInstance Is Nothing Then
					m_ProgramRepositoryInstance = TryCast(ObjectFactoryInstance.Create(Of IProgramRepository)(), ProgramRepository)
				End If

				Return m_ProgramRepositoryInstance
			End Get
		End Property

		<TestMethod>
		Public Sub GetProgramStoriesFilename()
			Dim expected As String = "ProgramStoriesList_18.xml"

			Dim actual As String = DataAccessUtility.GetProgramStoriesFilename("18")

			Assert.AreEqual(Of String)(expected, actual, "Filename was wrong.")
		End Sub

		<TestMethod>
		Public Sub AssertProgramStoriesFileDoesNotExistInStorageAtStartOfTest()
			Assert.IsFalse(UnitTestUtility.FileExists(DataAccessConstants.FilenameProgramInformation), "Program information file should not exist.")
		End Sub

		<TestMethod>
		Public Sub WhenNoCachedStoryListIsAvailableForTheRequestedProgramRefreshFromServer()
			Dim actual = RepositoryInstance.GetStories("18")

			Dim filename As String = DataAccessUtility.GetProgramStoriesFilename("18")

			Assert.IsTrue(UnitTestUtility.FileExists(filename), "Cached story does not exist.  Refresh() was probalby not called by repository.")
		End Sub

		<TestMethod>
		Public Sub WhenNoCachedStoryListIsAvailableReturnAnEmptyStoryList()
			Dim actual = RepositoryInstance.GetStories("18")

			Assert.IsNotNull(actual, "Should not be null.")
			Assert.IsNotNull(actual.Stories, "Stories collection was null.")
			Assert.AreEqual(Of Integer)(0, actual.Stories.Count, "Story count should be 0.")
		End Sub

		<TestMethod>
		Public Sub WhenCachedStoryListIsAvailableReturnListOfStories()
			RepositoryInstance.RefreshStories("18")

			Dim actual = RepositoryInstance.GetStories("18")

			Assert.IsNotNull(actual, "Should not be null.")
			Assert.IsNotNull(actual.Stories, "Stories collection was null.")
			Assert.AreNotEqual(Of Integer)(0, actual.Stories.Count, "Story count should not be 0.")
		End Sub

		<TestMethod>
		Public Sub WhenStoryListIsRefreshedFromServerTheyAreStoredToDisk()
			RepositoryInstance.RefreshStories("18")

			Assert.IsTrue(UnitTestUtility.FileExists(DataAccessUtility.GetProgramStoriesFilename("18")), "Cached file did not exist.")
		End Sub

		<TestMethod, Asynchronous, Tag("RefreshEvent")>
		Public Sub WhenRefreshedRepositoryRaisesRefreshEvent()
			' turn off test mode...this method will actually call NPR
			RepositoryInstance.TestModeConfiguration = Nothing

			Dim tester As New StoryRepositoryTester(RepositoryInstance)

			Dim programIdForAllThingsConsidered As String = "2"

            EnqueueCallback(Sub() RepositoryInstance.RefreshStories(programIdForAllThingsConsidered))

			EnqueueDelay(2000)

			EnqueueCallback(Sub() tester.AssertEvent(programIdForAllThingsConsidered))

			EnqueueTestComplete()
		End Sub

		<TestMethod>
		Public Sub GetStoryByProgramIdAndStoryId()
			RepositoryInstance.RefreshStories("18")

			Dim expectedStoryId As String = "131216486"
			Dim expectedProgramId As String = "18"

			Dim actual = RepositoryInstance.GetStory(expectedProgramId, expectedStoryId)

			Assert.IsNotNull(actual, "Story was null.")
			Assert.AreEqual(Of String)("In Israel, When Is A Jew Not Jewish Enough?", actual.Title, "Story title was wrong.")
		End Sub

		<TestMethod>
		Public Sub ObjectFactoryCanCreateInstanceOfIStory()
			Dim story = ObjectFactoryInstance.Create(Of IStory)()

			Assert.IsNotNull(story, "Value was null.")
		End Sub

		<TestMethod>
		Public Sub WhenStoryNotAvailableOnDiskReturnFileNotFoundStory()
			RepositoryInstance.RefreshStories("18")

			Dim bogusStoryId As String = "bogus story id"
			Dim expectedProgramId As String = "18"

			Dim actual = RepositoryInstance.GetStory(expectedProgramId, bogusStoryId)

			Assert.IsNotNull(actual, "Story was null.")

			Assert.AreEqual(Of String)("Story not found", actual.Title, "Title should say story not found.")
			Assert.AreEqual(Of String)(bogusStoryId, actual.Id, "Story id was wrong.")
		End Sub

		<TestMethod>
		Public Sub SaveCurrentStoryId()
			RepositoryInstance.SaveCurrentStoryId("asdf")

			Assert.IsTrue(UnitTestUtility.FileExists(DataAccessConstants.FilenameStoryInformation), "Story information file does not exist.")
		End Sub

		<TestMethod>
		Public Sub SaveCurrentStoryIdAcceptsNull()
			RepositoryInstance.SaveCurrentStoryId(Nothing)

			Assert.IsTrue(UnitTestUtility.FileExists(DataAccessConstants.FilenameStoryInformation), "Story information file does not exist.")
		End Sub

		<TestMethod>
		Public Sub GetCurrentStoryId()
			Dim expected As String = "asdf"

			RepositoryInstance.SaveCurrentStoryId(expected)

			Dim actual = RepositoryInstance.GetCurrentStoryId

			Assert.AreEqual(Of String)(expected, actual, "CurrentStoryId did not match.")
		End Sub

		<TestMethod>
		Public Sub GetCurrentStoryIdWhenValueIsEmptyStringReturnsNull()
            Dim expected As String = String.Empty

			RepositoryInstance.SaveCurrentStoryId(String.Empty)

			Dim actual = RepositoryInstance.GetCurrentStoryId

			Assert.AreEqual(Of String)(expected, actual, "CurrentStoryId did not match.")
		End Sub

		<TestMethod>
		Public Sub GetCurrentStoryIdWhenValueIsNullStringReturnsNull()
            Dim expected As String = String.Empty

			RepositoryInstance.SaveCurrentStoryId(Nothing)

			Dim actual = RepositoryInstance.GetCurrentStoryId

			Assert.AreEqual(Of String)(expected, actual, "CurrentStoryId did not match.")
		End Sub
	End Class
End Namespace
