Imports Microsoft.VisualStudio.TestTools.UnitTesting
Imports Benday.Npr.Phone.Presentation.Interfaces
Imports Benday.Npr.Phone.UnitTests.DataAccess
Imports Benday.Npr.Phone.Common.Repositories
Imports Benday.Npr.Phone.DataAccess
Imports Benday.Npr.Phone.Presentation
Imports Microsoft.Silverlight.Testing

Namespace Benday.Npr.Phone.UnitTests.Presentation
	<TestClass>
	Public Class StoryListViewModelFixture
		Inherits UnitTestBase

		Protected Overrides Sub OnTestInitialize()
			UnitTestUtility.WipeProgramDataInIsolatedStorage()
			m_ViewModelInstance = Nothing
			m_RepositoryInstance = Nothing
			m_ProgramRepositoryInstance = Nothing
			MyBase.OnTestInitialize()
		End Sub

		Private m_ViewModelInstance As IStoryListViewModel
		Public ReadOnly Property ViewModelInstance As IStoryListViewModel
			Get
				If m_ViewModelInstance Is Nothing Then
					m_ViewModelInstance = New StoryListViewModel(ProgramRepositoryInstance, RepositoryInstance, ObjectFactoryInstance)
				End If

				Return m_ViewModelInstance
			End Get
		End Property

		Private m_RepositoryInstance As StoryRepository
		Public ReadOnly Property RepositoryInstance As StoryRepository
			Get
				If m_RepositoryInstance Is Nothing Then
					m_RepositoryInstance = TryCast(ObjectFactoryInstance.Create(Of IStoryRepository)(), StoryRepository)

					m_RepositoryInstance.TestModeConfiguration = New StoryRepositoryTestModeConfiguration With {.TestData = UnitTestResources.AllThingsConsideredStories}
				End If

				Return m_RepositoryInstance
			End Get
		End Property

		Private m_ProgramRepositoryInstance As IProgramRepository
		Public ReadOnly Property ProgramRepositoryInstance As IProgramRepository
			Get
				If m_ProgramRepositoryInstance Is Nothing Then
					m_ProgramRepositoryInstance = New ProgramRepositoryFake
				End If

				Return m_ProgramRepositoryInstance
			End Get
		End Property

		<TestMethod>
		Public Sub CreateInstanceOfStoryListViewModelViaObjectFactory()
			Dim item = ObjectFactoryInstance.Create(Of IStoryListViewModel)()

			Assert.IsNotNull(item)
		End Sub

		<TestMethod>
		Public Sub LoadFromCache()
			' force the repository to write the test data to cache
			RepositoryInstance.RefreshStories("2")

			ViewModelInstance.Load()

			UnitTestUtility.AssertNotNullOrEmpty(ViewModelInstance.CurrentProgramId, "CurrentProgramId")
			UnitTestUtility.AssertNotNullOrEmpty(ViewModelInstance.CurrentProgramTitle, "CurrentProgramId")

			Assert.IsNotNull(ViewModelInstance.Stories, "Programs was null.")

			Assert.AreEqual(Of Integer)(10, ViewModelInstance.Stories.Items.Count, "Story count was wrong.")

			AssertStory(ViewModelInstance, 0, "In Israel, When Is A Jew Not Jewish Enough?", "131216486")
			AssertStory(ViewModelInstance, 1, "Economic Woes Spur Interest In Refinancing Plan", "131223118")
			AssertStory(ViewModelInstance, 9, "Deficit Panel Pushes Cuts To Social Security", "131223274")
		End Sub

		<TestMethod>
		Public Sub LoadForProgramWithoutLocalCache()
			ViewModelInstance.Load()

			UnitTestUtility.AssertNotNullOrEmpty(ViewModelInstance.CurrentProgramId, "CurrentProgramId")
			UnitTestUtility.AssertNotNullOrEmpty(ViewModelInstance.CurrentProgramTitle, "CurrentProgramId")

			Assert.IsNotNull(ViewModelInstance.Stories, "Programs was null.")

			Assert.AreEqual(Of Integer)(0, ViewModelInstance.Stories.Items.Count, "Program count was wrong.")
		End Sub

		<TestMethod, Asynchronous>
		Public Sub LoadForProgramWithoutLocalCacheCallsRefresh()
			' turn off test mode on the repository...this method will actually call NPR
			RepositoryInstance.TestModeConfiguration = Nothing

            ' calling this should cause the load to call refresh() 
            EnqueueCallback(Sub() ViewModelInstance.Load())

            EnqueueCallback(Sub() Assert.AreEqual(Of Integer)(0, ViewModelInstance.Stories.Items.Count, "Story count at start of test was wrong."))

			EnqueueDelay(3000)

            EnqueueCallback(Sub() Assert.AreEqual(Of Integer)(10, ViewModelInstance.Stories.Items.Count, "Story collection should be populated."))

			EnqueueTestComplete()
		End Sub

		<TestMethod, Asynchronous>
		Public Sub RefreshEventOnRepositoryPopulatesStories()
			' turn off test mode on the repository...this method will actually call NPR
			RepositoryInstance.TestModeConfiguration = Nothing

			ViewModelInstance.Load()

			Assert.IsNotNull(ViewModelInstance.Stories, "Programs was null.")

			Assert.AreEqual(Of Integer)(0, ViewModelInstance.Stories.Items.Count, "Story count at start of test was wrong.")

			' cause the story repository to refresh and throw a refresh event

            EnqueueCallback(Sub() RepositoryInstance.RefreshStories("2"))

			EnqueueDelay(3000)

            EnqueueCallback(Sub() Assert.AreEqual(Of Integer)(10, ViewModelInstance.Stories.Items.Count, "Story collection should be populated."))

			EnqueueTestComplete()
		End Sub

		<TestMethod>
		Public Sub SelectItem_TriggersUpdatedEvent_UpdatesSelectedItemId()
			' force the repository to write the test data to cache
			RepositoryInstance.RefreshStories("2")

			ViewModelInstance.Load()

			Assert.IsNotNull(ViewModelInstance.Stories, "Programs was null.")

			Assert.AreEqual(Of Integer)(10, ViewModelInstance.Stories.Items.Count, "Story count was wrong.")

			Assert.IsNull(ViewModelInstance.Stories.SelectedItem, "Nothing should be selected yet.")

			Assert.AreEqual(Of String)(Nothing, ViewModelInstance.CurrentStoryId, "Current story id value was wrong before selection.")

			Dim expectedSelection = ViewModelInstance.Stories.Items(0)

			Dim tester = New StoryListViewModelEventTester(ViewModelInstance)

			expectedSelection.IsSelected = True

			Assert.IsTrue(tester.GotSelectedEvent, "Should have gotten selection changed event.")

			Assert.AreEqual(Of String)(expectedSelection.Id, ViewModelInstance.CurrentStoryId, "Current story id value was wrong.")
		End Sub

		<TestMethod>
		Public Sub SelectItem_SavesCurrentStoryIdToIsolatedStorage()
			' force the repository to write the test data to cache
			RepositoryInstance.RefreshStories("2")
			ViewModelInstance.Load()

			Assert.IsNotNull(ViewModelInstance.Stories, "Stories was null.")

			SelectStoryAndAssertIdIsSavedToIsolatedStorage(ViewModelInstance, 0)
			SelectStoryAndAssertIdIsSavedToIsolatedStorage(ViewModelInstance, 1)
		End Sub

		<TestMethod>
		Public Sub WhenLoadMethodIsCalledCurrentProgramIdAndTitleIsLoaded()
			' program info is loaded via ProgramRepositoryFake
			ViewModelInstance.Load()

			UnitTestUtility.AssertNotNullOrEmpty(ViewModelInstance.CurrentProgramId, "CurrentProgramId")
			UnitTestUtility.AssertNotNullOrEmpty(ViewModelInstance.CurrentProgramTitle, "CurrentProgramTitle")
		End Sub

		<TestMethod>
		Public Sub WhenLoadMethodIsCalledCurrentStoryIdIsLoaded()
			' force the repository to write the test data to cache
			RepositoryInstance.RefreshStories("2")

			ViewModelInstance.Load()

			Assert.IsNull(ViewModelInstance.CurrentStoryId, "CurrentStoryId should be null.")

			SelectStoryAndAssertIdIsSavedToIsolatedStorage(ViewModelInstance, 0)

			Dim expectedCurrentStoryId = ViewModelInstance.CurrentStoryId

			m_ViewModelInstance = Nothing

			ViewModelInstance.Load()

			Assert.AreEqual(Of String)(expectedCurrentStoryId, ViewModelInstance.CurrentStoryId, "CurrentStoryId did not match expected value after call to Load.")
		End Sub

		Private Sub SelectStoryAndAssertIdIsSavedToIsolatedStorage(ByVal viewModel As IStoryListViewModel, ByVal index As Integer)
			Dim selectThis = viewModel.Stories.Items(index)

			selectThis.IsSelected = True

			Dim repository = ObjectFactoryInstance.Create(Of IStoryRepository)()

			Dim currentStoryId = repository.GetCurrentStoryId

			Assert.AreEqual(Of String)(selectThis.Id, currentStoryId, "Story Id did not match what was stored.")
		End Sub

		Private Sub AssertStory(ByVal instance As IStoryListViewModel, ByVal programIndex As Integer, ByVal expectedTitle As String, ByVal expectedId As String)
			Dim actual = instance.Stories.Items(programIndex)

			Assert.AreEqual(Of String)(expectedTitle, actual.Text, "Text")
			Assert.AreEqual(Of String)(expectedId, actual.Id, "Id")
		End Sub
	End Class
End Namespace
